@@ -21,13 +21,24 @@
 import os
 import errno
 import socket
+import Queue as queue
 
 import traceback
 
-from asynchia.util import EMPTY_BYTES
-from asynchia.const import trylater, connection_lost
+from asynchia.util import EMPTY_BYTES, is_unconnected, socketpair, b
+from asynchia.const import trylater, connection_lost, inprogress
+
+__version__ = '0.1.3'
+
+class CallSynchronized(object):
+    def __init__(self, handler, sock):
+        self.sock = sock
+        self.handler = handler
+    
+    def call(self, fun):
+        self.handler.funs.put(fun)
+        self.sock.send(b('a'))
 
-__version__ = '0.1.2'
 
 class SocketMapClosedError(Exception):
     pass
@@ -60,6 +71,18 @@ class SocketMap(object):
         self.notifier = notifier
         self.closed = False
     
+    def constructed(self):
+        wakeup, other = socketpair()
+        self.callsync = CallSynchronized(
+            _CallSynchronizedHandler(
+                SocketTransport(self, other)
+                ),
+            wakeup
+        )
+    
+    def call_synchronized(self, fun):
+        self.callsync.call(fun)
+    
     def poll(self, timeout=None):
         raise NotImplementedError
     
@@ -326,14 +349,9 @@ class SocketTransport(Transport):
             self.socket_map.del_transport(self)
         
         sock.setblocking(0)
-        try:
-            sock.getpeername()
-        except socket.error, err:
-            if err.args[0] == errno.ENOTCONN:
-                await = True
-                self.connected = False
-            else:
-                raise
+        if is_unconnected(sock):
+            await = True
+            self.connected = False
         else:
             self.connected = True
             # To provide consistency. If an async socket that already had
@@ -474,7 +492,7 @@ class SocketTransport(Transport):
             sockets and currently only implemented for IPv4 and IPv6.
             See arp(7) for details. """
         try:
-            return self.socket.send(data)
+            return self.socket.send(data, flags)
         except socket.error, err:
             if err.args[0] in trylater:
                 return 0
@@ -535,7 +553,7 @@ class SocketTransport(Transport):
         """ Connect to (host, port). """
         err = self.socket.connect_ex(address)
         # EWOULDBLOCK is only expected with WinSock.
-        if err in (errno.EINPROGRESS, errno.EWOULDBLOCK):
+        if err in inprogress:
             if not self.awaiting_connect:
                 self.connected = False
                 self.await_connect()
@@ -689,4 +707,22 @@ class Server(AcceptHandler):
         finally:
             self.transport.close()
 
+
+class _CallSynchronizedHandler(Handler):
+    """ Implementation detail. """
+    def __init__(self, transport, per_run=1):
+        Handler.__init__(self, transport)
+        self.transport.set_readable(True)
+        
+        self.funs = queue.Queue()
+        self.per_run = per_run
+    
+    def handle_read(self):
+        """ Implementation detail. """
+        for _ in self.transport.recv(self.per_run):
+            try:
+                self.funs.get_nowait()()
+            except queue.Empty:
+                break
+
 defaultsocket_factory = socket.socket
