@@ -146,12 +146,16 @@ class RockSolidSocketMap(ControlSocketSocketMap):
 
 class SelectSocketMap(FragileSocketMap):
     """ Decide which sockets have I/O to do using select.select. """
+    available = True
+    
     def __init__(self, notifier=None):
         FragileSocketMap.__init__(self, notifier)
         self.writers = []
         self.socket_list = []
         
         self.socket_list.append(self.controlreceiver)
+        
+        self.constructed()
     
     def add_transport(self, handler):
         """ See SocketMap.add_transport. """
@@ -236,15 +240,19 @@ class SelectSocketMap(FragileSocketMap):
         super(SelectSocketMap, self).close()
     
     def is_empty(self):
-        return bool(self.socket_list)
+        return not bool(self.socket_list)
 
 
 class PollSocketMap(RobustSocketMap):
     """ Decide which sockets have I/O to do using select.poll. 
     
-    Do not refer to this class without explicitely checking for its existance
-    first, it may not exist on some platforms (it is known not to on Windows).
+    Do not refer to this class without explicitely checking whether its
+    functionality is available on your operation system by checking
+    the `available` class-member, it may not exist on some platforms
+    (it is known not to on Windows).
     """
+    available = hasattr(select, 'poll')
+    
     def __init__(self, notifier=None):
         RobustSocketMap.__init__(self, notifier)
         self.socket_list = {}
@@ -252,6 +260,8 @@ class PollSocketMap(RobustSocketMap):
         
         self.controlfd = self.controlreceiver.fileno()
         self.poller.register(self.controlfd, select.POLLIN | select.POLLPRI)
+        
+        self.constructed()
     
     def add_transport(self, handler):
         """ See SocketMap.add_transport. """
@@ -345,16 +355,19 @@ class PollSocketMap(RobustSocketMap):
         super(PollSocketMap, self).close()
     
     def is_empty(self):
-        return bool(self.socket_list)
+        return not bool(self.socket_list)
 
 
 class EPollSocketMap(RockSolidSocketMap):
     """ Decide which sockets have I/O to do using select.epoll. 
     
-    Do not refer to this class without explicitely checking for its existance
-    first, it may not exist on some platforms (it is known not to on Windows
-    and BSD).
+    Do not refer to this class without explicitely checking whether its
+    functionality is available on your operation system by checking
+    the `available` class-member, it may not exist on some platforms
+    (it is known not to on Windows and BSD).
     """
+    available = hasattr(select, 'epoll')
+    
     def __init__(self, notifier=None):
         RockSolidSocketMap.__init__(self, notifier)
         self.socket_list = {}
@@ -362,6 +375,8 @@ class EPollSocketMap(RockSolidSocketMap):
         
         self.controlfd = self.controlreceiver.fileno()
         self.poller.register(self.controlfd, select.EPOLLIN | select.EPOLLPRI)
+        
+        self.constructed()
     
     def add_transport(self, handler):
         """ See SocketMap.add_transport. """
@@ -446,7 +461,7 @@ class EPollSocketMap(RockSolidSocketMap):
         super(EPollSocketMap, self).close()
     
     def is_empty(self):
-        return bool(self.socket_list)
+        return not bool(self.socket_list)
 
 
 # It is possible to only get hangup events by applying the hack presented
@@ -460,9 +475,13 @@ class EPollSocketMap(RockSolidSocketMap):
 class KQueueSocketMap(RockSolidSocketMap):
     """ Decide which sockets have I/O to do using select.kqueue. 
     
-    Do not refer to this class without explicitely checking for its existance
-    first, it may not exist on some platforms (it only exists on BSD).
+    Do not refer to this class without explicitely checking whether its
+    functionality is available on your operation system by checking
+    the `available` class-member, it may not exist on some platforms
+    (it only exists on BSD).
     """
+    available = hasattr(select, 'kqueue')
+    
     def __init__(self, nevents=100, notifier=None):
         RockSolidSocketMap.__init__(self, notifier)
         self.socket_list = {}
@@ -476,6 +495,8 @@ class KQueueSocketMap(RockSolidSocketMap):
                            select.KQ_FILTER_READ,
                            select.KQ_EV_ADD)], 0
         )
+        
+        self.constructed()
     
     def add_transport(self, handler):
         """ See SocketMap.add_transport. """
@@ -569,22 +590,10 @@ class KQueueSocketMap(RockSolidSocketMap):
         super(KQueueSocketMap, self).close()
     
     def is_empty(self):
-        return bool(self.socket_list)
+        return not bool(self.socket_list)
 
 
-DefaultSocketMap = SelectSocketMap
-
-if hasattr(select, 'poll'):
-    DefaultSocketMap = PollSocketMap
-else:
-    del PollSocketMap
-
-if hasattr(select, 'epoll'):
-    DefaultSocketMap = EPollSocketMap
-else:
-    del EPollSocketMap
-
-if hasattr(select, 'kqueue'):
-    DefaultSocketMap = KQueueSocketMap
-else:
-    del KQueueSocketMap
+order = [SelectSocketMap, PollSocketMap, EPollSocketMap, KQueueSocketMap]
+for mp in order:
+    if mp.available:
+        DefaultSocketMap = mp
